library crazy_db;
import 'dart:io';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:sqflite/sqflite.dart';
export 'package:sqflite/sqflite.dart';

class CrazyDB {
  static const String _tag = "crazy_db";
  static final CrazyDB _instance = CrazyDB();
  static late String dbPath;
  static late Database db;
  static bool _isTransaction = false;
  static List<String> _transactionSqlList = [];

  String _table = "";
  List<String> _joinList = [];
  List<String> _whereList = [];
  List<String> _orWhereList = [];
  List<String> _orderByList = [];
  List<String> _groupByList = [];
  List<String> _havingList = [];

  String _sql = "";

  static CrazyDB getInstance() {
    return _instance;
  }

  static Future<String> _copyDbToDirectory({String? db}) async {
    db = db ?? "crazyDB.sqlite";

    String applicationDir = await getDatabasesPath();
    CrazyDB.dbPath = "$applicationDir/$db";
    debugPrint("${CrazyDB._tag} 获取沙盒文件夹路径=>  ${CrazyDB.dbPath}");
    File file = File(CrazyDB.dbPath);
    if (await file.exists() == false) {
      //写入
      ByteData bytes = await rootBundle.load('assets/$db');
      File(CrazyDB.dbPath).writeAsBytes(
          bytes.buffer.asUint8List(bytes.offsetInBytes, bytes.lengthInBytes));
    }
    //打开数据库
    await openDB();

    return CrazyDB.dbPath;
  }

  static Future<bool> openDB() async {
    CrazyDB.db = await openDatabase(CrazyDB.dbPath);
    return CrazyDB.db.isOpen;
  }

  static void closeDB() async {
    CrazyDB.db.close();
  }

  static Future<List<Map<String, Object?>>> selectRaw(
      {required String sql}) async {
    return await CrazyDB.db.rawQuery(sql);
  }

  static Future<int> insertRaw({required String sql}) async {
    return await CrazyDB.db.rawInsert(sql);
  }

  static Future<int> updateRaw({required String sql}) async {
    return await CrazyDB.db.rawUpdate(sql);
  }

  static Future<int> deleteRaw({required String sql}) async {
    return await CrazyDB.db.rawDelete(sql);
  }

  //、连接数据库
  static Future<CrazyDB> connection(String db) async {
    await CrazyDB._copyDbToDirectory(db: db);

    return CrazyDB();
  }

  static transaction(){
    CrazyDB._isTransaction = true;
    CrazyDB._transactionSqlList = [];
  }

  static commit() async {
    try {
      Batch batch = CrazyDB.db.batch();
      for (String sql in CrazyDB._transactionSqlList) {
        String headAction = sql.substring(0,sql.indexOf(" "));
        if(headAction == "update"){
          batch.rawUpdate(sql);
        }else if(headAction == "insert"){
          batch.rawInsert(sql);
        }else if(headAction == "delete"){
          batch.rawDelete(sql);
        }
      }
      await batch.commit(noResult: true);
      CrazyDB.rollback();
    } catch (e) {
      debugPrint("${CrazyDB._tag} Exception=> $e");
      throw Exception(e);
    }
  }

  static rollback() async{
    CrazyDB._isTransaction = false;
    CrazyDB._transactionSqlList = [];
  }
  static Future<int> _transactionStart(String sql){
    CrazyDB._transactionSqlList.add(sql);
    return Future(()=>-1);
  }

  static CrazyDB table(String table) {
    CrazyDB crazyDB = CrazyDB.getInstance();
    crazyDB._table = table;
    crazyDB._sql = "";
    crazyDB._joinList = [];
    crazyDB._whereList = [];
    crazyDB._orWhereList = [];
    crazyDB._orderByList = [];
    crazyDB._groupByList = [];
    crazyDB._havingList = [];

    return crazyDB;
  }

  CrazyDB select({List? list}) {
    if (_sql.contains("select") == false) {
      _sql = "select";
      if (list == null) {
        _sql = "select *";
      } else {
        _sql = "$_sql ${list.join(",")}";
      }

      _sql = "$_sql from $_table ";
    }

    return this;
  }

  CrazyDB _field(List list) {
    int index = _sql.indexOf("from");
    if (index != -1) {
      String end = _sql.substring(index);
      _sql = "select ${list.join(",")} $end";
    }

    return this;
  }

  Future<int> create(Map<String, dynamic> map) async{
    List keys = [];
    List values = [];
    map.forEach((key, value) {
      keys.add(key);
      values.add(value is String ? "'$value'" : value);
    });

    _sql =
        "insert into $_table(${keys.join(',')}) values(${values.join(',')});";
    
    return CrazyDB._isTransaction? CrazyDB._transactionStart(_sql):CrazyDB.db.rawInsert(_sql);
  }

  CrazyDB join(
      String tab2, String tab1Column, String symbol, String tab2Column) {
    _joinList.add("inner join $tab2 ON $tab1Column $symbol $tab2Column");
    return this;
  }

  CrazyDB leftJoin(
      String tab2, String tab1Column, String symbol, String tab2Column) {
    _joinList.add("left join $tab2 ON $tab1Column $symbol $tab2Column");
    return this;
  }

  CrazyDB rightJoin(
      String tab2, String tab1Column, String symbol, String tab2Column) {
    _joinList.add("right join $tab2 ON $tab1Column $symbol $tab2Column");
    return this;
  }

  CrazyDB where({required String? column, String? symbol, required value}) {
    symbol = symbol ?? "=";
    if (value is String) {
      _whereList.add("$column $symbol '$value'");
    } else {
      _whereList.add("$column $symbol $value");
    }

    return this;
  }

  CrazyDB whereNotNull({required String column}) {
    _whereList.add("$column is not null");
    return this;
  }

  CrazyDB whereNull({required String column}) {
    _whereList.add("$column is null");
    return this;
  }

  CrazyDB orWhere({required String column, String? symbol, required value}) {
    symbol = symbol ?? "=";
    if (value is String) {
      _orWhereList.add("$column $symbol '$value'");
    } else {
      _orWhereList.add("$column $symbol $value");
    }

    return this;
  }

  CrazyDB whereBetween(String column, List list) {
    if (list.length > 1) {
      _whereList.add("$column between ${list[0]} and ${list[1]}");
    }
    return this;
  }

  CrazyDB whereNotBetween(String column, List list) {
    if (list.length > 1) {
      _whereList.add("$column not between ${list[0]} and ${list[1]}");
    }
    return this;
  }

  CrazyDB whereIn(String column, List list) {
    if (list.isNotEmpty) {
      _whereList.add("$column in(${list.join(',')})");
    }
    return this;
  }

  CrazyDB whereNotIn(String column, List list) {
    if (list.isNotEmpty) {
      _whereList.add("$column not in(${list.join(',')})");
    }
    return this;
  }

  Future<int> update(Map<String, dynamic> map) {
    _sql = "UPDATE $_table";
    List temp = [];
    map.forEach((key, value) {
      value = value is String ? "'$value'" : value;
      temp.add("set $key=$value");
    });
    _sql = "$_sql ${temp.join(',')}";

    _sql = _formatWhereToSql();

    return CrazyDB._isTransaction? CrazyDB._transactionStart(_sql):CrazyDB.updateRaw(sql: "$_sql;");
  }

  Future<int> delete() {
    _sql = "delete from $_table";
    _sql = _formatWhereToSql();
    return CrazyDB._isTransaction? CrazyDB._transactionStart(_sql):CrazyDB.deleteRaw(sql: "$_sql;");
  }

  Future<List<Map<String, Object?>>> get({List? list}) {
    select(list: list);
    _sql = _formatWhereToSql();
    return CrazyDB.selectRaw(sql: "$_sql;");
  }

  Future<Map<String, Object?>> first() async {
    select();
    _sql = _formatWhereToSql();
    List list = await CrazyDB.selectRaw(sql: "$_sql limit 0,1 ;");
    return list.isNotEmpty ? list[0] : {};
  }

  Future<Map<String, Object?>> find(int id) {
    return where(column: "id", value: id).first();
  }

  Future<List<Map<String, Object?>>> paginate(int page, {int? perPage}) {
    select();
    perPage = perPage ?? 10;
    _sql = _formatWhereToSql();
    return CrazyDB.selectRaw(
        sql: "$_sql limit ${(page - 1) * perPage},$perPage ;");
  }

  Future<int?> count({String? column}) async {
    select();
    column = column ?? "*";
    _sql = _formatWhereToSql();
    int index = _sql.indexOf("from");
    if (index != -1) {
      String end = _sql.substring(index);
      _sql = "select count($column) $end";
    }

    return Sqflite.firstIntValue(await CrazyDB.selectRaw(sql: "$_sql;"));
  }

  Future<List<Map<String, Object?>>> sum(
      {String? column, String? alias}) async {
    select();
    column = column ?? "";
    alias = alias != null ? "as $alias" : "";
    _sql = _formatWhereToSql();
    int index = _sql.indexOf("from");
    if (index != -1) {
      String start = _sql.substring(0, index);
      String end = _sql.substring(index);
      _sql = "$start,sum($column) $alias $end";
    }
    return await CrazyDB.selectRaw(sql: "$_sql;");
  }

  Future<int> avg({String? column}) async {
    select();
    column = column ?? "";
    _sql = _formatWhereToSql();
    int index = _sql.indexOf("from");
    if (index != -1) {
      String end = _sql.substring(index);
      _sql = "select avg($column) $end";
    }

    return Sqflite.firstIntValue(await CrazyDB.selectRaw(sql: "$_sql;")) ?? 0;
  }

  Future<dynamic> pluck(String column, {String? key}) async {
    select();
    if (column.isNotEmpty) {
      key == null ? _field([column]) : _field([column, key]);
    }
    groupBy(column);
    List list = await get();

    if (key == null) {
      List temp = [];
      for (Map item in list) {
        temp.add(item[column]);
      }
      return temp;
    } else {
      Map temp = {};
      for (Map item in list) {
        temp[item[key]] = item[column];
      }
      return temp;
    }
  }

  Future<dynamic> value(String column) async {
    select();
    _field([column]);
    Map map = await first();
    return map.isNotEmpty ? map[column] : null;
  }

  CrazyDB orderBy(String column, {String? soft}) {
    soft = soft ?? "";
    _orderByList.add("$column $soft");
    return this;
  }

  CrazyDB orderByDesc(String column) {
    _orderByList.add("$column desc");
    return this;
  }

  CrazyDB groupBy(String column) {
    _groupByList.add(column);
    return this;
  }

  CrazyDB having(String sql) {
    _havingList.add(sql);
    return this;
  }

  static String getSql() {
    debugPrint("${CrazyDB._tag} sql=>${CrazyDB._instance._sql}");
    return CrazyDB._instance._sql;
  }

  String _formatWhereToSql() {
    String join = _joinToSql();
    if (join.isNotEmpty) {
      _sql = "$_sql $join";
    }

    String where = _whereToSql();
    if (where.isNotEmpty) {
      _sql = "$_sql $where";
    }

    String orWhere = _orWhereToSql();
    if (orWhere.isNotEmpty) {
      _sql = "$_sql $orWhere";
    }

    String groupBy = _groupByToSql();
    if (groupBy.isNotEmpty) {
      _sql = "$_sql $groupBy";
    }

    String orderBy = _orderByToSql();
    if (orderBy.isNotEmpty) {
      _sql = "$_sql $orderBy";
    }

    String having = _havingToSql();
    if (having.isNotEmpty) {
      _sql = "$_sql $having";
    }

    return _sql;
  }

  String _joinToSql() {
    if (_joinList.isNotEmpty) {
      return _joinList.join(" ");
    }
    return "";
  }

  String _whereToSql() {
    if (_whereList.isNotEmpty) {
      String where = "";
      for (int i = 0; i < _whereList.length; i++) {
        String sql = _whereList[i];
        if (i == 0) {
          where = "where $sql";
        } else {
          where = "$where and $sql";
        }
      }
      return where;
    }
    return "";
  }

  String _orWhereToSql() {
    if (_orWhereList.isNotEmpty) {
      String orWhere = "";
      for (int i = 0; i < _orWhereList.length; i++) {
        String sql = _orWhereList[i];
        if (i == 0) {
          orWhere = "or $sql";
        } else {
          orWhere = "$orWhere and $sql";
        }
      }
      return orWhere;
    }
    return "";
  }

  String _orderByToSql() {
    if (_orderByList.isNotEmpty) {
      return "order by ${_orderByList.join(',')}";
    }
    return "";
  }

  String _groupByToSql() {
    if (_groupByList.isNotEmpty) {
      return "group by ${_groupByList.join(',')}";
    }
    return "";
  }

  String _havingToSql() {
    if (_havingList.isNotEmpty) {
      return "having ${_havingList.join(' and ')}";
    }
    return "";
  }
}